'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

#font color: 183,0,0

import os
import pickle
import pygame

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))

try:
    pygame.mixer.init()
except:
    print "Could not initialize sound."
    SND = False


class NoneSound:
    def play(self): pass


def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(data_dir, filename), mode)

def load_image(filename, colorkey=None):
    path = os.path.join(data_dir, "gfx", filename)
    return pygame.image.load(path).convert()

def load_sound(filename):
    path = os.path.join(data_dir, "sfx", filename)
    try:
        return pygame.mixer.Sound(path)
    except:
        print "Could not load sound: ", path
    return NoneSound()

def set_music(filename):
    path = os.path.join(data_dir, "sfx", filename)
    pygame.mixer.music.load(path)


def load_gfx(filenames):
    """ return dict of graphics """
    dc_gfx = {}
    for name in filenames:
        dc_gfx[name] = load_image(name, colorkey=(0, 255, 0, 255))
        dc_gfx[name].set_colorkey((0,255,0,255))
    return dc_gfx

def load_game_gfx():
    """ load game graphics """
    filenames = [
        "bg.png",
        "mcgrumpington.png",
        "deathblossom.png",
        "bullet.png",
        "header.png",
        "sheep.png",
        "zombie01.png",
        "zombie02.png",
        "zombie03.png",
        "zombie04.png",
        "zombie05.png",
        "zombie06.png",
        "zombie07.png",
        "zombie08.png",
        "zombie09.png",
        "zombie10.png",
        "medkit1.png",
        "medkit2.png",
        "ammo1.png",
        "ammo2.png",
    ]
    return load_gfx(filenames)

def load_menu_gfx():
    """ load main menu graphics """
    filenames = []
    return load_gfx(filenames)

def load_highscore(filename):
    try:
        f = open(filepath(filename),"rb")
        score = pickle.load(f)
        f.close()
    except:
        f.close()
        score = {}
    return score

def save_highscore(score, filename="highscore"):
    try:
        f = open(filepath(filename),"wb")
        pickle.dump(score, f, 2)
        f.close()
    except:
        f.close()

def load_grid(filename, width=32):
    img = load_image(filename)
    sw, sh = img.get_size()

    x = 0
    images = []
    surf = pygame.Surface((width, sh))
    surf.set_colorkey((0,255,0,255))
    while x < sw:
        surf.blit(img, (-x,0))
        images.append(surf.copy())
        x += width
    return images

def load_perk_imgs(perk):
    for name in perk.keys():
        fname = perk[name]["img"]
        path_t = os.path.join(data_dir, "gfx", "perks", fname+"_t.png")
        path_p = os.path.join(data_dir, "gfx", "perks", fname+"_p.png")
        perk[name]["img_t"] = pygame.image.load(path_t).convert()
        perk[name]["img_p"] = pygame.image.load(path_p).convert()

def load_moan_sfx():
    filenames = [
        "zombiegroan01.ogg",
        "zombiegroan03.ogg",
        "zombiegroan05.ogg",
        "zombiegroan08.ogg",
        "zombiegroan09.ogg",
        "zombiegroan10.ogg",
    ]
    return load_sfx(filenames)

def load_hurt_sfx():
    filenames = [
        "hurt1.ogg",
        "hurt2.ogg",
        "hurt3.ogg",
    ]
    return load_sfx(filenames)

def load_zhit_sfx():
    filenames = [
        "zombiehit01.ogg",
        "zombiehit02.ogg",
        "zombiehit03.ogg",
        "zombiehit04.ogg",
    ]
    return load_sfx(filenames)

def load_main_sfx():
    filenames = [
        "automaticshotgunfire.ogg",
        "automaticshotgunreload.ogg",
        "jackhammerfire.ogg",
        "jackhammerreload1.ogg",
        "jackhammerreload2.ogg",
        "playerdeath.ogg",
        "pumpactionshotgunfire.ogg",
        "pumpactionshotgunreload.ogg",
        "zombiesheep.ogg",
        "zombiegroan11.ogg",
        "levelup.ogg",
        "munchmunch.ogg",
        "pickingupammo.ogg",
        "pickingupmedkit.ogg",
    ]
    sfx = load_sfx(filenames)
    firesounds = [
        "automaticshotgunfire.ogg",
        "jackhammerfire.ogg",
        "pumpactionshotgunfire.ogg",
    ]
    for name in firesounds:
        sfx[name].set_volume(0.5)
    return sfx

def load_sfx(filenames):
    sounds = {}
    for name in filenames:
        sounds[name] = load_sound(name)
    return sounds


SND_MAIN = load_main_sfx()
SND_MOAN = load_moan_sfx()
SND_HURT = load_hurt_sfx()
SND_ZHIT = load_zhit_sfx()
