import data
import pygame
import random

from pygame.locals import *

import utils
from gui import *
from people import *
from engine import State
from perks import perk
from data import SND_MAIN, SND_MOAN
random.seed()
SH_MODE = [False]
HIGHSCORE = data.load_highscore("highscore")


def gen_perks():
    utils.gen_next(perk)
    data.load_perk_imgs(perk)


class Game(State):
    def __init__(self):
        State.__init__(self)
        pygame.key.set_repeat(20,50)
        Zombie.hp_max = 5
        Zombie.dam = 2
        self.screen = pygame.display.get_surface()
        self.screen.fill((0,0,0))
        self.killcount = 0

        #zombies
        self.zombie_timer = utils.Timer(100)
        self.zombie_spawn = 1
        self.max_zombies = 150
        self.ztrack = 9
        self.zombie_wave = 0
        self.moan_timer = utils.Timer(random.randint(20,60))
        self.zombie_sndlist = SND_MOAN

        #load graphics
        self.dc_gfx = data.load_game_gfx()

        #set bg
        self.bg = self.dc_gfx["bg.png"]
        self.lawn = self.screen.subsurface([0,35,800,565])
        self.update_bg()

        #spritegroups
        self.bullets = pygame.sprite.Group()
        self.powerups = pygame.sprite.Group()
        self.on_bottom = pygame.sprite.Group()
        self.zombies = pygame.sprite.Group()
        self.on_top = pygame.sprite.Group()
        self.all_sprites = pygame.sprite.Group()

        #generate zombie imagelist
        self.zombie_namelist = []

        for i in xrange(1, 11):
            nr = str(i)
            if len(nr) == 1:
                nr = "0"+nr
            name = "zombie"+ nr +".png"
            self.zombie_namelist.append(name)

        if SH_MODE == [True]:
            self.zombie_namelist = ["sheep.png"]
            self.zombie_sndlist = {"sheep": SND_MAIN["zombiesheep.ogg"]}
            self.zombie_spawn = 7
            self.moan_timer.set_interval(40)

        #cursor
        animlist = data.load_grid("reload.png")
        self.cursor = AnimCursor(self.dc_gfx["deathblossom.png"], animlist)
        pos = utils.center((60,60), self.lawn.get_size())
        pygame.mouse.set_pos(pos)
        self.on_top.add(self.cursor)
        self.hover_hp = HoverHP()

        #player
        self.player = Player(self.dc_gfx["mcgrumpington.png"], self.cursor)
        self.player.center(self.screen)
        self.all_sprites.add(self.player)
        self.gen_zombies(self.zombie_spawn)

        data.set_music("bgm.xm")
        pygame.mixer.music.play(-1)

    def update_bg(self):
        self.screen.fill((0,0,0))
        self.screen.blit(self.dc_gfx["header.png"], (0,0))
        self.lawn.blit(self.bg, (0,0))
        pygame.display.flip()

    def update_screen(self):
        self.lawn.blit(self.bg, (0,0))
        self.on_bottom.update()
        self.on_bottom.draw(self.lawn)
        self.all_sprites.update()
        self.all_sprites.draw(self.lawn)
        self.on_top.update()
        self.on_top.draw(self.lawn)
        pygame.display.flip()

    def gen_zombies(self, nr):
        znr = random.randint(nr, nr+5)
        for i in xrange(znr):
            name = random.choice(self.zombie_namelist)
            z = Zombie(self.dc_gfx[name], self.player)
            self.zombies.add(z)
            self.all_sprites.add(z)

        self.zombie_wave += 1
        if self.zombie_wave > self.ztrack:  #zombie levelup
            if Zombie.hp_max < 1000:
                Zombie.hp_max += 2
            if Zombie.dam < 25:
                Zombie.dam += 1
            if self.zombie_spawn < 100:
                self.zombie_spawn += 1
            self.zombie_wave = 0

    def gen_powerup(self, pos):
        type = random.randint(0, 1)
        if type:
            images=[self.dc_gfx["ammo1.png"],self.dc_gfx["ammo2.png"]]
        else:
            images=[self.dc_gfx["medkit1.png"],self.dc_gfx["medkit2.png"]]
        powerup = PowerUp(images, pos, type)
        self.on_bottom.add(powerup)
        self.powerups.add(powerup)

    def shoot(self):
        if self.player.shoot(self.cursor) and not self.player.reload_timer.active:
            x, y = pygame.mouse.get_pos()
            min_x = x - self.player.burst
            max_x = x + self.player.burst
            min_y = y - self.player.burst
            max_y = y + self.player.burst

            angle = self.player.angle
            pos = self.player.rect.center

            burst = []
            for i in xrange(self.player.burst):
                max_x += 5
                x = random.randint(min_x, max_x)
                y = random.randint(min_y, max_y)
                b = Bullet(self.dc_gfx["bullet.png"],(x,y),angle, pos,
                    self.player.bullet_speed)
                burst.append(b)

            self.bullets.add(burst)
            self.all_sprites.add(burst)
        elif not self.player.cooldown_timer.active:
            self.player.reload()

    def check_collisions(self):
        #player hit by zombie
        phit = pygame.sprite.spritecollide(self.player, self.zombies, 0)
        if phit:
            for z in phit:
                if z.hitting():
                    if random.random() > self.player.dodge:
                        dam = -z.dam + self.player.ac
                        if dam < 0:
                            self.player.add_hp(dam)
                            self.player.stun()
                            if random.random() < self.player.hitback:
                                z.gets_hit(self.player.mdam, True)
                                if self.player.stun_chance < random.random():
                                    z.stun(50)
                                if random.random() < self.player.killing_blow or \
                                    z.dead:
                                        self.player.add_ep(10)##TODO: ep calc
                                        z.kill()
                                        self.killcount += 1

        #zombie hit by bullet
        zhit = pygame.sprite.groupcollide(self.zombies,self.bullets,0,0)
        if zhit:
            for zombie in zhit:
                bullets = zhit[zombie]
                for bullet in bullets:
                    if zombie.hit_flag and bullet.hit_flag:
                        continue

                    mindam = self.player.dam
                    zombie.gets_hit(random.randint(mindam, mindam*2))
                    self.killcount += 1
                    bullet.set_hit()
                    if random.random() > self.player.piercing:
                        bullet.kill()

                if zombie.dead:
                    if random.random() < self.player.drop_rate:
                        self.gen_powerup(zombie.get_position())
                    zombie.kill()
                    self.player.add_ep(zombie.ep) #ep

        #player touches powerup
        collect=pygame.sprite.spritecollide(self.player,self.powerups,0)
        if collect:
            for p in collect:
                if p.type:
                    self.player.add_ammo()
                else:
                    self.player.add_hp()
                p.picked_up()

        #player hovers powerup and has telekinesis
        hover = pygame.sprite.spritecollide(self.cursor, self.powerups, 0)
        if self.player.telekinesis and hover:
            for p in hover:
                if not p.tele_timer.active:
                    p.hover = True
                    p.tele_timer.activate()
                else:
                    p.hover = True
                    if p.tele_timer.update():
                        if p.type:
                            self.player.add_ammo()
                        else:
                            self.player.add_hp()
                        p.picked_up()

        #player hovers zombie and is doctor
        hover = pygame.sprite.spritecollide(self.cursor, self.zombies, 0)
        if hover and self.player.see_zhp:
            zombie = hover[0]#-1?
            if zombie is not self.hover_hp.zombie:
                self.hover_hp.activate(zombie)
                self.on_top.add(self.hover_hp)
        else:
            self.hover_hp.deactivate()


    def pause(self):
        self.paused = True
        self.next = Pause(self.player.req_tree, self.player.level)
        pygame.mixer.pause()
        self.quit()

    def unpause(self):
        State.unpause(self)
        self.update_bg()
        self.player.shooting = False  #reset input events
        pressed = pygame.key.get_pressed()
        self.player.set_keypress(0,pressed[K_UP])
        self.player.set_keypress(1,pressed[K_DOWN])
        self.player.set_keypress(2,pressed[K_LEFT])
        self.player.set_keypress(3,pressed[K_RIGHT])

        for b in self.bullets:
            b.unpause()
        pygame.mixer.unpause()
        self.check_events()
        self.player.score_draw()

    def check_events(self):
        for ev in pygame.event.get():
            if ev.type==QUIT:
                self.close_game()
            elif ev.type==MOUSEBUTTONDOWN:
                #todo: left/right button?
                self.player.set_shooting(True)
            elif ev.type==MOUSEBUTTONUP:
                #todo: left/right button?
                self.player.set_shooting(False)
            elif ev.type==KEYDOWN:
                k = ev.key
                if k==K_w:
                    self.player.set_keypress(0, 1) #up
                elif k==K_s:
                    self.player.set_keypress(1, 1) #down
                elif k==K_a:
                    self.player.set_keypress(2, 1) #left
                elif k==K_d:
                    self.player.set_keypress(3, 1) #right
                elif k==K_r:
                    self.player.reload()

            elif ev.type==KEYUP:
                k = ev.key
                if k==K_ESCAPE:
                    self.pause()
                elif k==K_w:
                    self.player.set_keypress(0, 0) #up
                elif k==K_s:
                    self.player.set_keypress(1, 0) #down
                elif k==K_a:
                    self.player.set_keypress(2, 0) #left
                elif k==K_d:
                    self.player.set_keypress(3, 0) #right
        pygame.event.pump()


    def tick(self):
        if self.player.hp < 1:
            self.gameover()

        if self.player.shooting:
            self.shoot()

        if self.zombie_timer.update():
            if len(self.zombies) < self.max_zombies:
                self.gen_zombies(self.zombie_spawn)

        if self.moan_timer.update():
            if not pygame.mixer.get_busy():
                name = random.choice(self.zombie_sndlist.keys())
                self.zombie_sndlist[name].play()
                if not SH_MODE==[True]:
                    self.moan_timer.set_interval(random.randint(20,60))

        if self.player.levelup:
            self.player.levelup = False
            p = self.player.new_perk()
            self.pause()
            if self.player.level==22:  #Zombies respawn quicker
                self.ztrack = 4
            elif self.player.level==40:
                self.ztrack = 1
                self.zombie_timer.set_interval(80)
            elif self.player.level == 50:
                self.zombie_timer.set_interval(60)
            elif self.player.level == 60:
                self.zombie_timer.set_interval(40)
            elif self.player.level == 70:
                self.zombie_timer.set_interval(20)

            self.next = NewPerk(p)

        self.clock.tick(20)

    def gameover(self):
        self.paused = False
        score, level = self.player.get_highscore_data()
        self.next = GameOver(self.killcount, score, level)
        pygame.mixer.music.stop()
        pygame.key.set_repeat()
        pygame.mixer.fadeout(1)
        self.quit()



class NewPerk(State):
    def __init__(self, name="Scavenger"):
        State.__init__(self)
        self.screen = pygame.display.get_surface()
        alpha_bg = self.screen.copy()
        alpha_bg.set_alpha(99)
        self.bg = pygame.Surface((800,600))
        self.bg.fill((0,0,0))
        self.bg.blit(alpha_bg, (0,0))
        self.perk = perk[name]


        gfx = data.load_gfx(["newperk.png", "deathblossom.png",
        "donateblood.png"])
        deco = gfx["donateblood.png"]

        img = gfx["newperk.png"]
        pos = utils.center(img.get_size(), (800,600))
        self.bg.blit(img, pos)

        img = self.perk["img_t"]
        x = utils.center(img.get_size(),(800,600))[0]
        self.bg.blit(img, (x, pos[1]+50))

        img = self.perk["img_p"]
        x = utils.center(img.get_size(),(800,600))[0]
        self.bg.blit(img, (x, pos[1]+75))

        img = TextArea(self.perk["desc"])
        x = utils.center(img.get_size(),(800,600))[0]
        self.bg.blit(img, (x, pos[1]+110))

        img = data.load_image("pause.png")
        self.bg.blit(img, (335, 525))


        self.update_scr = True
        cursor = Cursor(gfx["deathblossom.png"])
        self.sprites = pygame.sprite.RenderUpdates()
        self.sprites.add(cursor)
        self.cont = ClickArea([334,408,125,25], (320,408), deco, (463, 408))

    def main_start(self):
        SND_MAIN["levelup.ogg"].play()
        pygame.event.clear()

    def update_screen(self):
        self.sprites.update()
        self.screen.blit(self.bg, (0,0))
        if not self.update_scr:
            changed = self.sprites.draw(self.screen)
            pygame.display.update(changed)
        else:
            self.update_scr = False
            changed = self.sprites.draw(self.screen)
            pygame.display.flip()

    def check_events(self):
        pos = pygame.mouse.get_pos()
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYUP:
                k = ev.key
                if k==K_ESCAPE:
                    self.quit()
            elif ev.type == MOUSEBUTTONDOWN:
                if self.cont.collidepoint(pos):
                    self.bg.set_alpha(0)
                    self.update_scr = True
                    self.quit()
                    return

            if self.cont.collidepoint(pos):
                self.cont.highlight(1)
            else:
                self.cont.highlight(0)

    def update_screen(self):
        self.sprites.update()
        self.screen.blit(self.bg, (0,0))
        if not self.update_scr:
            changed = self.sprites.draw(self.screen)
            pygame.display.update(changed)
        else:
            self.update_scr = False
            changed = self.sprites.draw(self.screen)
            pygame.display.flip()



class Pause(State):
    def __init__(self, req_tree, level):
        State.__init__(self)
        self.screen = pygame.display.get_surface()
        self.req_tree = req_tree
        self.level = level

        alpha_bg = self.screen.copy()
        alpha_bg.set_alpha(99)
        self.bg = pygame.Surface((800,600))
        self.bg.fill((0,0,0))
        self.bg.blit(alpha_bg, (0,0))

        img_names=["escmenu.png","deathblossom.png","donateblood.png",
            "pause.png"]
        images = data.load_gfx(img_names)

        img = data.load_image("pause.png")
        self.bg.blit(img, (335, 525))
        self.deco = images["donateblood.png"]
        self.menu = images["escmenu.png"]
        self.menu_pos = utils.center(self.menu.get_size(), (800,600))
        self.bg.blit(self.menu, self.menu_pos)
        self.cursor = Cursor(images["deathblossom.png"])

        self.sprites = pygame.sprite.RenderUpdates()
        self.sprites.add(self.cursor)
        self.update_scr = True

        self.cont = ClickArea([311,217,178,25],(325,220),self.deco, (459,220))
        self.perk = ClickArea([311,278,178,25],(310,280),self.deco, (476,280))
        self.surr = ClickArea([311,339,178,25],(315,342),self.deco, (470,342))
        self.buttons=[self.cont, self.perk, self.surr]

    def unpause(self):
        State.unpause(self)
        self.update_scr = True

    def update_screen(self):
        self.sprites.update()
        self.screen.blit(self.bg, (0,0))
        if not self.update_scr:
            changed = self.sprites.draw(self.screen)
            pygame.display.update(changed)
        else:
            self.update_scr = False
            changed = self.sprites.draw(self.screen)
            pygame.display.flip()

    def check_events(self):
        pos = pygame.mouse.get_pos()
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYDOWN:
                k = ev.key
                if k==K_ESCAPE:
                    self.quit()
            elif ev.type == MOUSEBUTTONDOWN:
                if self.cont.collidepoint(pos):
                    self.quit()
                elif self.perk.collidepoint(pos):
                    self.next = PerkBoard(self.req_tree, self.level)
                    self.pause()
                    self.quit()
                elif self.surr.collidepoint(pos):
                    self.kill_prev = True
                    self.next = StartMenu()
                    pygame.mixer.music.stop()
                    self.quit()

        for button in self.buttons:
            if button.collidepoint(pos):
                button.highlight(1)
            elif button.hover:
                button.highlight(0)

class PerkBoard(State):
    def __init__(self, req_tree, level=1):
        State.__init__(self)
        self.screen = pygame.display.get_surface()
        self.bg = self.screen.copy()
        images = data.load_gfx(["deathblossom.png"])
        self.cursor = Cursor(images["deathblossom.png"])
        self.sprites = pygame.sprite.RenderUpdates()
        self.sprites.add(self.cursor)

        img = data.load_image("perkboard.png")
        self.offset = utils.center(img.get_size(), (800,600))
        self.bg.blit(img, self.offset)

        img = TextLine("Level: "+str(level))
        self.bg.blit(img, (600,400))

        self.highlight = []
        for p in req_tree.keys():
            if req_tree[p] is True:
                ppos = perk[p]["pos"]
                x = self.offset[0] + ppos[0]
                y = self.offset[1] + ppos[1]
                self.bg.blit(perk[p]["img_p"],(x,y))
                self.highlight.append(ToolTipArea([x,y,32,32], p))
        self.tooltip = ToolTip()
        self.update_scr = True

    def check_events(self):
        pos = pygame.mouse.get_pos()
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYUP:
                k = ev.key
                if k==K_ESCAPE:
                    self.quit()
            elif ev.type == MOUSEBUTTONDOWN:
                self.quit()

        for h in self.highlight:
            if h.collidepoint(pos):
                x, y = h.center
                if x > 500:
                    x = pos[0] - 200
                if y > 432:
                    y = pos[1] - 100
                self.tooltip.activate(h.name, (x,y))
                break
            else:
                self.tooltip.active = False

    def update_screen(self):
        self.sprites.update()
        self.screen.blit(self.bg, (0,0))
        if not self.update_scr:
            changed = self.sprites.draw(self.screen)
            if self.tooltip.active:
                self.screen.blit(self.tooltip, self.tooltip.pos)
            pygame.display.flip()
        else:
            self.update_scr = False
            pygame.display.flip()


class StartMenu(State):
    def __init__(self):
        State.__init__(self)
        self.screen = pygame.display.get_surface()
        gfx = data.load_gfx([
            "donateblood.png",
            "titlescreen.png",
            "deathblossom.png",
            "cursor3.png"])
        self.bg = gfx["titlescreen.png"]
        self.deco = gfx["donateblood.png"]

        self.sprites = pygame.sprite.RenderUpdates()
        cursor = Cursor(gfx["cursor3.png"])
        self.sprites.add(cursor)

        self.start = ClickArea([15,15,212,30], (230,15), self.deco)
        self.quitgame = ClickArea([604,559,184,30], (582,563), self.deco)
        self.help = ClickArea([12,560,80,27], (100,563), self.deco)
        self.highscore = ClickArea([602,13,188,30], (580,15), self.deco)
        self.crd = pygame.rect.Rect([385,352,112,116])

        self.buttons=[self.start,self.quitgame,self.help,self.highscore]
        self.update_scr = True

    def main_start(self):
        pygame.event.clear()

    def update_screen(self):
        self.sprites.update()
        self.screen.blit(self.bg, (0,0))
        if not self.update_scr:
            changed = self.sprites.draw(self.screen)
            pygame.display.update(changed)
        else:
            self.update_scr = False
            pygame.display.flip()

    def check_events(self):
        pos = pygame.mouse.get_pos()
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYUP:
                k = ev.key
                if k==K_ESCAPE:
                    self.quit()
            elif ev.type == MOUSEBUTTONDOWN:
                if self.start.collidepoint(pos):
                    self.bg.set_alpha(0)
                    self.update_scr = True
                    self.next = Game()
                    self.quit()
                    return
                elif self.highscore.collidepoint(pos):
                    self.next = Highscore()
                    self.quit()
                elif self.help.collidepoint(pos):
                    self.next = Help()
                    self.quit()
                elif self.quitgame.collidepoint(pos):
                    self.quit()
                elif self.crd.collidepoint(pos):
                    self.next = Credits()
                    self.quit()

        for button in self.buttons:
            if button.collidepoint(pos):
                button.highlight(1)
            elif button.hover:
                button.highlight(0)

class Highscore(State):
    def __init__(self):
        State.__init__(self)
        self.update_scr = True
        self.screen = pygame.display.get_surface()
        gfx = data.load_gfx(["highscore.png", "donateblood.png",
            "cursor3.png"])
        self.bg = gfx["highscore.png"]
        self.sprites = pygame.sprite.RenderUpdates()
        cursor = Cursor(gfx["cursor3.png"])
        self.sprites.add(cursor)
        self.next = StartMenu()
        deco = gfx["donateblood.png"]
        self.back = ClickArea([300,565,196,27],(285,565),deco, (495,565))

        lines = HIGHSCORE.keys()
        lines.sort(reverse=True)

        pos = (50,145)
        x, y = pos
        for line in lines:
            t = TextLine(str(HIGHSCORE[line]["name"]))
            self.bg.blit(t, (x,y))
            t2 = TextLine(str(HIGHSCORE[line]["level"]).strip())
            self.bg.blit(t2, (x+270,y))
            t3 = HighscoreText(line)
            self.bg.blit(t3, (x+450,y))

            y += 40

    def main_start(self):
        pygame.event.clear()

    def update_screen(self):
        self.sprites.update()
        self.screen.blit(self.bg, (0,0))
        if not self.update_scr:
            changed = self.sprites.draw(self.screen)
            pygame.display.update(changed)
        else:
            self.update_scr = False
            pygame.display.flip()

    def check_events(self):
        pos = pygame.mouse.get_pos()
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYUP:
                k = ev.key
                if k==K_ESCAPE:
                    self.quit()
            elif ev.type == MOUSEBUTTONDOWN:
                if self.back.collidepoint(pos):
                    self.quit()

        if self.back.collidepoint(pos):
            self.back.highlight(1)
        elif self.back.hover:
            self.back.highlight(0)

class Credits(State):
    def __init__(self):
        State.__init__(self)
        self.next = StartMenu()
        self.screen = pygame.display.get_surface()
        self.gfx = data.load_gfx(["credits.png", "cursor3.png",
            "creditssh.png"])

        self.bg = self.gfx["credits.png"]
        cursor = Cursor(self.gfx["cursor3.png"])
        self.sprites = pygame.sprite.RenderUpdates()
        self.sprites.add(cursor)
        self.crd = pygame.rect.Rect([375,497,50,50])

        if SH_MODE[0]:
            self.bg.blit(self.gfx["creditssh.png"],(371,493))
        self.update_scr = True

    def main_start(self):
        pygame.event.clear()

    def draw_crd(self):
        self.bg.blit(self.gfx["creditssh.png"],(371,493))
        self.update_scr = True
        SND_MAIN["munchmunch.ogg"].play()
        SH_MODE[0] = True

    def update_screen(self):
        self.sprites.update()
        self.screen.blit(self.bg, (0,0))
        if not self.update_scr:
            changed = self.sprites.draw(self.screen)
            pygame.display.update(changed)
        else:
            self.update_scr = False
            self.sprites.draw(self.screen)
            pygame.display.flip()

    def check_events(self):
        pos = pygame.mouse.get_pos()
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYUP:
                self.quit()
            elif ev.type == MOUSEBUTTONDOWN:
                if self.crd.collidepoint(pos) and not SH_MODE[0]:
                    self.draw_crd()
                else:
                    self.quit()

class EnterHighscore(State):
    def __init__(self, score="00000001", level="1"):
        State.__init__(self)
        self.next = StartMenu()
        self.screen = pygame.display.get_surface()
        alpha_bg = self.screen.copy()
        alpha_bg.set_alpha(99)

        self.bg = pygame.Surface((800,600))
        self.bg.fill((0,0,0))
        self.bg.blit(alpha_bg, (0,0))

        img = data.load_image("newhighscore.png")
        pos = utils.center(img.get_size(), (800,600))
        self.bg.blit(img, pos)

        self.level = level

        self.score = score
        self.score_img = TextLine(score)
        self.score_img.set_position((470,250))

        self.entry = TextEntry()
        self.entry.set_position((190,250))
        self.entry.gen_image()

        self.update_scr = True
        self.play_snd = True

    def main_start(self):
        pygame.event.clear()

    def update_screen(self):
        if self.update_scr:
            self.update_scr = False
            self.screen.blit(self.bg, (0,0))
            self.score_img.redraw()
            self.entry.redraw()
            pygame.display.flip()

    def check_events(self):
        pos = pygame.mouse.get_pos()
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYUP:
                k = ev.key
                if k==K_ESCAPE:
                    self.quit()
            elif ev.type==KEYDOWN:
                k = ev.key
                if k==K_BACKSPACE:
                    self.entry.rem_char()
                elif k==K_RETURN:
                    self.submit_score()
                elif 96 < k < 123 or 47 < k < 58:  #letter or number
                    char = pygame.key.name(k).upper()
                    self.entry.add_char(char)

    def submit_score(self):
        if not self.entry.text or self.entry.text=="_":
            return

        if not self.entry.full:
            name = self.entry.text[:-1]
        else:
            name = self.entry.text



        HIGHSCORE[self.score] = {
            "name" : name,
            "level" : self.level
        }

        l = HIGHSCORE.keys()
        l.sort()
        HIGHSCORE.pop(l[0])

        data.save_highscore(HIGHSCORE)
        self.next = Highscore()
        self.quit()

    def tick(self):
        State.tick(self)
        if self.play_snd:
            SND_MAIN["zombiegroan11.ogg"].play()
            self.play_snd = False

class GameOver(State):
    def __init__(self, killcount, score, level):
        State.__init__(self)
        self.next = StartMenu()
        self.screen = pygame.display.get_surface()

        alpha_bg = self.screen.copy()
        alpha_bg.set_alpha(99)
        self.bg = pygame.Surface((800,600))
        self.bg.fill((0,0,0))
        self.bg.blit(alpha_bg, (0,0))


        img = data.load_image("gameover.png")
        pos = utils.center(img.get_size(), (800,600))
        self.bg.blit(img, pos)
        img = TextLine("You killed "+str(killcount)+" zombies.", 30)
        pos = utils.center(img.get_size(), (800,600))
        self.bg.blit(img, (pos[0], 325))

        scorelist = HIGHSCORE.keys()
        scorelist.sort()
        for s in scorelist:
            if score > s:
                self.next = EnterHighscore(score, level)
                break

        self.update_scr = True

    def main_start(self):
        pygame.event.clear()
        SND_MAIN["playerdeath.ogg"].play()

    def update_screen(self):
        if self.update_scr:
            self.update_scr = False
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()

    def check_events(self):
        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYDOWN:
                self.quit()


class Help(State):
    def __init__(self):
        State.__init__(self)
        self.next = StartMenu()
        self.screen = pygame.display.get_surface()
        self.bg = data.load_image("help.png")
        self.update_scr = True

    def main_start(self):
        pygame.event.clear()

    def update_screen(self):
        if self.update_scr:
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()

    def check_events(self):
          for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYDOWN or ev.type==MOUSEBUTTONDOWN:
                self.quit()



class Logo(State):
    def __init__(self):
        State.__init__(self)
        self.next = StartMenu()

        self.screen = pygame.display.get_surface()
        self.screen.fill((0,0,0))

        self.logo = data.load_image("logo.png")
        self.alpha = 0
        self.logo.set_alpha(0)
        self.logopos = utils.center(self.logo.get_size(), (800,600))

        self.fadeout = False
        self.anim_timer = utils.Timer(1)

        data.set_music("logovoices.ogg")
        pygame.mixer.music.play()
        self.update_scr = True

    def update_screen(self):
        if self.update_scr:
            self.update_scr = False
            self.screen.fill((0,0,0))
            self.screen.blit(self.logo, self.logopos)
            pygame.display.flip()

    def tick(self):
        self.clock.tick(20)
        if self.anim_timer.update():
            if not self.fadeout:
                self.alpha += 1
                self.logo.set_alpha(self.alpha)
                self.update_scr = True
                if self.alpha > 255:
                    self.fadeout = True
            else:
                self.alpha -= 4
                self.logo.set_alpha(self.alpha)
                self.update_scr = True
                if self.alpha < -100:
                    self.quit()

    def check_events(self):
          for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            elif ev.type == KEYUP:
                self.quit()

    def quit(self):
        pygame.mixer.music.stop()
        State.quit(self)
